#include <u.h>
#include <libc.h>
#include <bio.h>
#include "xml.h"

void
xml2(Biobuf *bp, char *path, Elem *ep)
{
	Attr *ap;
	char *s;

	for(; ep; ep = ep->next){
		if(ep->pcdata)
			Bprint(bp, "%s/%s=%q\n", path, ep->name, ep->pcdata);
		else
			Bprint(bp, "%s/%s\n", path, ep->name);
		for(ap = ep->attrs; ap; ap = ap->next)
			Bprint(bp, "%s/%s/%s=%q\n", path, ep->name, ap->name, ap->value);
		if(ep->child){
			if((s = smprint("%s/%s", path, ep->name)) == nil)
				sysfatal("no memory %r\n");
			xml2(bp, s, ep->child);
			free(s);
		}
	
	}

}

static void
usage(void)
{
	fprint(2, "usage: %s [-w] [-n] [file.xml]\n", argv0);
	fprint(2, "  -w  don't strip leading and trailing whitespace\n");
	fprint(2, "  -n  don't strip namespace specifiers\n");
	exits("usage");
}

void
main(int argc, char **argv)
{
	Xml *xp;
	Biobuf bo;
	int i, fd, flags;

	flags = Fcrushwhite|Fstripnamespace;
	ARGBEGIN{
	case 'd':
		xml_debug++;
		break;
	case 'w':
		flags &= ~Fcrushwhite;		/* keep whitespace */
		break;
	case 'n':
		flags &= ~Fstripnamespace;	/* keep namespaces */
		break;
	}ARGEND

	fmtinstall('q', quotestrfmt);

	Binit(&bo, 1, OWRITE);
	if(argc == 0){
		if((xp = xml_parse(0, 8192, flags)) == nil)
			sysfatal("%s:%r\n", argv[0]);
		xml2(&bo, "", xp->root);
		xml_free(xp);
	}
	else
	for(i = 0; i < argc; i++){
		if((fd = open(argv[i], OREAD)) == -1){
			fprint(2, "%s: %s cannot open %r\n", argv0, argv[i]);
			continue;
		}
		if((xp = xml_parse(fd, 8192, flags)) == nil)
			sysfatal("%s:%r\n", argv[0]);
		close(fd);
		xml2(&bo, "", xp->root);
		xml_free(xp);

	}
	Bterm(&bo);
	exits(nil);
}
